unit Entity;

// Entity-Translator
// Version 1.0.3
// http://www.philo.de/xml/entity/
//
// Copyright 1999-2003 by
//   Dieter Khler
//   email: service@philo.de
//   web-site: http://www.philo.de/
//
// LICENSE
//
// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
// the specific language governing rights and limitations under the License.
//
// The Original Code is "Entity.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/"). Portions created by the Initial Developer
// are Copyright (C) 1999-2003 Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"), in which case the
// provisions of the GPL are applicable instead of those above. If you wish to
// allow use of your version of this file only under the terms of the GPL, and
// not to allow others to use your version of this file under the terms of the
// MPL, indicate your decision by deleting the provisions above and replace them
// with the notice and other provisions required by the GPL. If you do not delete
// the provisions above, a recipient may use your version of this file under the
// terms of any one of the MPL or the GPL.

interface

uses
  {$ifdef VER100} dsgnintf {$else} typinfo {$endif} // In D5, or above, EPropertyError is located in TypInfo.pas
  ,sysutils, classes;

type
  TCharToEntityTranslator = class (TComponent)
  private
    FDictionary: TStrings;
    FFilename: TFileName;
    FNames: TStrings;
    FValues: TStrings;
    function GetFile: TFileName;
    function GetHasEntries: Boolean;
    function IsWhiteSpace(const S: Char): boolean;
    function ReplaceString(const value: string): string;
    procedure SetFile(const Filename: TFileName);
  public
    constructor Create(aOwner: TComponent); override;
    destructor  Destroy; override;
    function Translate(const source: string): string; virtual;
    function ReplaceWhiteSpace(const Source: string;
                               const ReplaceChar: Char): string; virtual;
    function ReplaceChar(const Source: string;
                         const FindChar,
                               ReplaceChar: Char): string; virtual;
    function ReplaceCharByString(const Source: string;
                                 const FindChar: Char;
                                 const ReplaceStr: string): string;
    property HasEntries: Boolean read GetHasEntries;
  published
    property DictionaryFile: TFileName read GetFile write SetFile;
  end;

type
  TEntityToCharTranslator = class (TCharToEntityTranslator)
  private
    FLeftDelimiter: char;
    FRightDelimiter: char;
    function GetLeftDelimiter: Char;
    procedure SetLeftDelimiter(const value: Char);
    function GetRightDelimiter: Char;
    procedure SetRightDelimiter(const value: Char);
  public
    constructor Create(aOwner: TComponent); override;
    function translate(const source: string): string; override;
  published
    property LeftDelimiter: Char read GetLeftDelimiter write SetLeftDelimiter default '&';
    property RightDelimiter: Char read GetRightDelimiter write SetRightDelimiter default ';';
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('XML',[TCharToEntityTranslator,TEntityToCharTranslator]);
end;




//++++++++++++++++++++++ TCharToEntityTranslator +++++++++++++++++++++++

constructor TCharToEntityTranslator.Create(aOwner: TComponent);
begin
  Inherited Create(aOwner);
  FDictionary:= TStringList.create;
  FNames:= TStringList.create;
  FValues:= TStringList.create;
end;

destructor  TCharToEntityTranslator.Destroy;
begin
  FDictionary.free;
  FNames.free;
  FValues.free;
  Inherited destroy;
end;

function TCharToEntityTranslator.GetFile: TFileName;
begin
  Result:= FFilename;
end;

function TCharToEntityTranslator.GetHasEntries: Boolean;
begin
  Result:= FDictionary.Count > 0;
end;

function TCharToEntityTranslator.ReplaceString(const value: string): string;
var
  i: integer;
begin
  Result:= value;
  for i:= 0 to FNames.count-1 do
    if AnsiCompareStr(value,FNames[i]) = 0 then begin
      Result:= FValues[i];
      break;
    end;
end;

procedure TCharToEntityTranslator.SetFile(const Filename: TFileName);
var
  i,p: longint;
begin
  if Filename <> FFilename then
  begin
    FFilename:= Filename;
    try
      FDictionary.LoadFromFile(Filename);
      FNames.clear;
      FValues.clear;
      for i:= 0 to FDictionary.count-1 do begin
        p:= Pos('=',FDictionary[i]);
        if p > 0 then begin
          FNames.add(copy(FDictionary[i],1,p-1));
          FValues.add(copy(FDictionary[i],p+1,length(FDictionary[i])-p));
        end;
      end; {for i:= 0 ...}
    except
      On EFOpenError do
      begin
        FDictionary.clear;
      end;
    end;
  end;
end;

function TCharToEntityTranslator.IsWhiteSpace(const S: Char): boolean;
begin
  Case ord(S) of
    $09,$0A,$0D,$20:
    result:= true;
  else
    result:= false;
  end;
end;

function TCharToEntityTranslator.translate(const source: string): string;
var
  i: longint;
begin
  Result:= '';
  for i:= 1 to length(source) do
    Result:= concat(Result,ReplaceString(source[i]));
end;

function TCharToEntityTranslator.ReplaceWhiteSpace(const Source: string;
                                                   const ReplaceChar: Char): string;
var
  i: integer;
begin
  Result:= Source;
  for i:= 1 to length(Result) do
    if IsWhiteSpace(Result[i])
      then Result:= concat(copy(Result,1,i-1),ReplaceChar,copy(Result,i+1,length(Result)-i));
end;

function TCharToEntityTranslator.ReplaceChar(const Source: string;
                                             const FindChar,
                                                   ReplaceChar: Char): string;
var
  i: integer;
begin
  Result:= Source;
  for i:= 1 to length(Result) do
    if Result[i] = FindChar
      then Result:= concat(copy(Result,1,i-1),ReplaceChar,copy(Result,i+1,length(Result)-i));
end;

function TCharToEntityTranslator.ReplaceCharByString(const Source: string;
                                                     const FindChar: Char;
                                                     const ReplaceStr: string): string;
var
  i: integer;
begin
  Result:= Source;
  for i:= 1 to length(Result) do
    if Result[i] = FindChar
      then Result:= concat(copy(Result,1,i-1),ReplaceStr,copy(Result,i+1,length(Result)-i));
end;



//++++++++++++++++++++ TEntityToCharTranslator +++++++++++++++++++++++++

constructor TEntityToCharTranslator.Create(aOwner: TComponent);
begin
  Inherited Create(aOwner);
  LeftDelimiter:= '&';
  RightDelimiter:= ';';
end;

function  TEntityToCharTranslator.GetLeftDelimiter: Char;
begin
  Result:= FLeftDelimiter;
end;

procedure TEntityToCharTranslator.SetLeftDelimiter(const value: Char);
begin
  if Length(value)<> 1
    then raise EPropertyError.create('Invalid value for left delimiter.')
    else FLeftDelimiter:= value;
end;

function  TEntityToCharTranslator.GetRightDelimiter: Char;
begin
  Result:= FRightDelimiter;
end;

procedure TEntityToCharTranslator.SetRightDelimiter(const value: Char);
begin
  if Length(value)<> 1
    then raise EPropertyError.create('Invalid value for right delimiter.')
    else FRightDelimiter:= value;
end;

function TEntityToCharTranslator.translate(const source: string): string;
var
  EntityAnfang,EntityEnde: longint;
  S: string;
begin
  Result:= '';
  S:= source;
  while length(S) > 0 do  {Solange S nicht abgearbeitet wurde, wiederhole:}
  begin
    if copy(S,1,1) = FLeftDelimiter then {beginnt S mit einer Entity?}
    begin
      EntityEnde := pos(RightDelimiter,S);
      if EntityEnde = 0 then {Falls Entity unvollstndig, ...}
      begin
        Result:= concat(Result,S); {... S als Teil-String verzeichnen}
        S:='';                     {S ist nun leer}
      end
      else {if EntityEnde = 0} {Falls Entity vollstndig, ...}
      begin
        {... nur die Entity als Teil-String verzeichnen:}
        Result:= concat(Result,ReplaceString(copy(S,1,EntityEnde)));
        {S um die Entity verkrzen:}
        S:= copy(S,EntityEnde+1,length(S));
      end; {if EntityEnde = 0}
    end
    else {if copy(s,1,1) = Left} {Keine Entity am Anfang der Zeile:}
    begin
      {Feststellen, wo die nchste Entity beginnt:}
      EntityAnfang := pos(FLeftDelimiter,S);
      {Gibt es noch mindestens eine Entity in S? ...}
      if EntityAnfang = 0 then
      begin                        {... Nein}
        Result:= concat(Result,S); {S verzeichnen}
        S:='';                     {S ist nun leer}
      end
      else {if EntityAnfang = 0}   {... Ja}
      begin
        {Teilstring vor der Entity verzeichnen:}
        Result:= concat(Result,copy(S,1,EntityAnfang-1));
        {S entsprechend krzen:}
        S:= copy(S,EntityAnfang,length(S));
      end; {if EntityAnfang = 0}
    end; {if copy(S,1,1) = Left}
  end; {while length(S)>0}
end;

end.
